local AssignSoldier = ...

AssignSoldier:registerBehaviorTreeNode({
    Name = _assignSoldierPrefix .. "_NODE",
    VariableList = {
		Var1 = "BEHAVIOR_TREE_DATA_AGENT",
		Var2 = "BEHAVIOR_TREE_DATA_WAIT",
		Var3 = "BEHAVIOR_TREE_DATA_LOCATION",
		GuardingLocation = "BEHAVIOR_TREE_DATA_LOCATION",
		IntermediateLocationUp = "BEHAVIOR_TREE_DATA_LOCATION",
		IntermediateLocationDown = "BEHAVIOR_TREE_DATA_LOCATION",
		Var4 = "BEHAVIOR_TREE_DATA_RESOURCE_TRANSPORT",
		Var5 = "BEHAVIOR_TREE_DATA_LOOP"
    },
    Init = function(self, instance)
		AssignSoldier:log("INIT!")
    end,
    Update = function(self, level, instance)
		AssignSoldier:log("UPDATE!")
		return self:updateFunc(level, instance)
    end,
    Finish = function(self, instance)
		AssignSoldier:log("FINISH!")
    end,
	updateFunc = function(self, level, instance)
		AssignSoldier:log("UPDATE FUNC!")
		AssignSoldier:log("Agent data: " .. tostring(self.Var1))
		AssignSoldier:log(tostring(self.Var1.Agent))
		AssignSoldier:log("Agent data owner: " .. tostring(self.Var1.Agent:getOwner()))

-- just for trial		self.Var1.Agent:setVehicle("VEHICLE_HORSE")
                local workplace = self.Var1.Agent:getOwner():getEnabledComponent("COMP_VILLAGER"):getJobInstance().Workplace
		-- No workplace found that matches this job
		if (workplace == nil or not workplace:isEnabled()) then
			self.Var1.Agent:setBehaviorMessage(foundation.localize("I_CANT_FIND_A_PLACE_TO_WORK"))
			return BEHAVIOR_TREE_NODE_RESULT.FALSE
		end
                local accessComponent = workplace:getOwner():findFirstObjectWithComponentDown("ASSIGN_SOLDIER_PATH_AEND_COMP")
                if (accessComponent == nil) then
			self.Var1.Agent:setBehaviorMessage(foundation.localize("I_CANT_FIND_A_PLACE_TO_WORK"))
			return BEHAVIOR_TREE_NODE_RESULT.FALSE
		end
                local _guardingLocationComp = accessComponent:getOwner():findFirstObjectWithComponentDown("ASSIGN_SOLDIER_ACCESS_GUARDING_LOCATION_COMP")
                if _guardingLocationComp == nil then       
			self.Var1.Agent:setBehaviorMessage(foundation.localize("I_CANT_FIND_A_PLACE_TO_WORK"))
			return BEHAVIOR_TREE_NODE_RESULT.FALSE
		end
                self.GuardingLocation:setDestination(_guardingLocationComp:getOwner(), { 0, 0 }, true)
                local _intermediateLocationUpComp = accessComponent:getOwner():findFirstObjectWithComponentDown("ASSIGN_SOLDIER_ACCESS_INTERMEDIATE_LOCATION_UP_COMP")
                if _intermediateLocationUpComp == nil then       
			self.Var1.Agent:setBehaviorMessage(foundation.localize("I_CANT_FIND_A_PLACE_TO_WORK"))
			return BEHAVIOR_TREE_NODE_RESULT.FALSE
		end
                self.IntermediateLocationUp:setDestination(_intermediateLocationUpComp:getOwner(), { 0, 0 }, true)
                local _intermediateLocationDownComp = accessComponent:getOwner():findFirstObjectWithComponentDown("ASSIGN_SOLDIER_ACCESS_INTERMEDIATE_LOCATION_DOWN_COMP")
                if _intermediateLocationDownComp == nil then       
			self.Var1.Agent:setBehaviorMessage(foundation.localize("I_CANT_FIND_A_PLACE_TO_WORK"))
			return BEHAVIOR_TREE_NODE_RESULT.FALSE
		end
                self.IntermediateLocationDown:setDestination(_intermediateLocationDownComp:getOwner(), { 0, 0 }, true)

                AssignSoldier:log("Wait data: " .. tostring(self.Var2))
		AssignSoldier:log("Location data: " .. tostring(self.Var3))
		AssignSoldier:log("Location data destination: " .. tostring(self.Var3.DestinationPosition))
		AssignSoldier:log("Resource transport data: " .. tostring(self.Var4))
		AssignSoldier:log("Loop data: " .. tostring(self.Var5))
		return BEHAVIOR_TREE_NODE_RESULT.TRUE
	end,
})

AssignSoldier:registerBehaviorTree({
	Id = _assignSoldierPrefix .. "_BEHAVIOR_TREE",
	VariableList = {
		{
			Name = "AgentData",
			DataType = "BEHAVIOR_TREE_DATA_AGENT",
			IsPublic = true,
			InitialValue = {} -- Optional if empty, leaves the default values (if any)
		},
		{
			Name = "DoJobTimer",
			DataType = "BEHAVIOR_TREE_DATA_WAIT",
			IsPublic = false,
			InitialValue = {
				TimeToWait = 30.0,
				Animation = AGENT_ANIMATION.GUARD,
				SetIdleAfterWait = false,
			}
		},
		{
			Name = "WorkplacePosition",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION",
			IsPublic = false,
			InitialValue = {
				CanNavigateOnGround = true,
				CanNavigateOnWater = true,
				IsSetOrientationOnDestination = true,
			}
		},
		{
			Name = "GuardingLocation",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION"
		},
		{
			Name = "IntermediateLocationUp",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION"
		},
		{
			Name = "IntermediateLocationDown",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION"
		},
		{
			Name = "TransportData",
			DataType = "BEHAVIOR_TREE_DATA_RESOURCE_TRANSPORT",
			IsPublic = false,
		},
		{
			Name = "WorkLoop",
			DataType = "BEHAVIOR_TREE_DATA_LOOP",
			IsPublic = true,
			InitialValue = {
				LoopCount = 1,
				Duration = 0,
				IsInfinite = false,
				IsDuration = false,
			}
		},
	},
	Root = {
		Name = "WorkingGlobalSequencer",
		Type = "SEQUENCER",
		Children = {
			{
				Name = "NeedFilledSelector",
				Type = "SELECTOR",
				Children = {
					{
						Name = "NeedFilledInverter",
						Type = "INVERTER",
						Child = {
							Name = "IsMyNeedFilled",
							Type = "IS_WORKPLACE_NEED_RESOURCE",
							AgentData = "AgentData",
						}
					},
					{
						Name = "FetchResourceSequencer",
						Type = "SEQUENCER",
						Children = {
							{
								Name = "FindResourceForWorkplace",
								Type = "FIND_RESOURCE_FOR_WORKPLACE",
								AgentData = "AgentData",
								TransportData = "TransportData",
							},
							{
								Name = "Transport",
								Type = "BEHAVIOR_TRANSPORT",
								AgentData = "AgentData",
								TransportData = "TransportData",
							},
						},
					},
				},
			},
			{
				Name = _assignSoldierPrefix .. "_NODE_INSTANCE",
				Type = _assignSoldierPrefix .. "_NODE",
				Var1 = "AgentData",
				Var2 = "DoJobTimer",
				Var3 = "WorkplacePosition",
				Var4 = "TransportData",
				Var5 = "WorkLoop",
                                GuardingLocation = "GuardingLocation",
                                IntermediateLocationUp = "IntermediateLocationUp",
                                IntermediateLocationDown = "IntermediateLocationDown"
			},
			{
			        Name = "GoToIntermediateLocationDown",
		                Type = "GO_TO",
		                AgentData = "AgentData",
		                Destination = "IntermediateLocationDown",
		        },
		        {
		                Name = "GoToIntermediateLocationUp",
		                Type = "GO_TO",
		                AgentData = "AgentData",
		                Destination = "IntermediateLocationUp",
                        },
			{
				Name = "WorkLoopRepeater",
				Type = "REPEAT",
				RepeatData = "WorkLoop",
				Child = {
					Name = "WorkLoopSequencer",
					Type = "SEQUENCER",
					Children = {
--[[			                        {
				                        Name = "SetupWork",
				                        Type = "SETUP_WORK",
				                        AgentData = "AgentData",
				                        TimeToWait = "DoJobTimer",
				                        WorkPosition = "WorkplacePosition",
			                        },
			                        {
				                        Name = "GoToResource",
				                        Type = "GO_TO",
				                        AgentData = "AgentData",
				                        Destination = "WorkplacePosition",
				                        BuildingPathType = "",
				                        AnimationData = "",
				                        AnimationSpeedMultiplier = "",
			                        },
--]]
			                        {
				                        Name = "GoToLocation",
				                        Type = "GO_TO",
				                        AgentData = "AgentData",
				                        Destination = "GuardingLocation",
			                        },
			                        {
				                        Name = "SetRotationLocation",
				                        Type = "SET_ORIENTATION",
				                        AgentData = "AgentData",
				                        Destination = "GuardingLocation",
			                        },
						{
							Name = "ProduceResource",
							Type = "PRODUCE_RESOURCE",
							AgentData = "AgentData",
							TimeToWait = "DoJobTimer",
						},
						{
							Name = "GettingXp",
							Type = "GIVE_JOB_XP",
							AgentData = "AgentData",
							ShouldReceiveXp = "",
						}
					}
				}
			},
		        {
		                Name = "GoToIntermediateLocationUp2",
		                Type = "GO_TO",
		                AgentData = "AgentData",
		                Destination = "IntermediateLocationUp",
                        },
			{
			        Name = "GoToIntermediateLocationDown2",
		                Type = "GO_TO",
		                AgentData = "AgentData",
		                Destination = "IntermediateLocationDown",
		        }
		}
	}
})